--[[
/run for _, v in pairs({pcall(dofile, "Interface/loginxml/characterselect.lua")}) do print(tostring(v)) end
]]

local Auto_Login_Path = "vx2019_AutoLogin\\AutoLoginData.lua"

local character_iterator = function()
	local index = 1
	
	return function()	
		local obj = _G["CharacterSelectCharacterButton"..index.."ButtonTextName"]
		if obj then
			local text = obj:GetText()
			
			if text and text ~= "" then
				index = index + 1
				return text:lower()
			end
		else
			return nil
		end
	end
end

CHARACTER_FACING_INCREMENT = 2;
CHARACTER_ROTATION_CONSTANT = 0.6
CHARACTER_SELECT_ROTATION_START_X = nil;

CHARACTERSELECT_REGION_SELECTED = nil;
CHARACTERSELECT_RECOVER_DELETE = nil;
MAX_REGION_BUTTONS = 15;
MAX_CHARACTERS_DISPLAYED = 8;
MAX_CHARACTERS_PER_REALM = 10;

CHARACTER_SELECT = {};

LOADLISTCOLOR = {};
LOADLISTCOLOR[1] = { r = 0, g = 1, b = 0 };
LOADLISTCOLOR[2] = { r = 1, g = 1, b = 1 };
LOADLISTCOLOR[3] = { r = 0.94, g = 0.4, b = 0.13 };
LOADLISTCOLOR[4] = { r = 1, g = 0, b = 0 };

function CharacterSelect_OnLoad(this)
	CHARACTER_SELECT.createIndex = 0;
	CHARACTER_SELECT.selectedIndex = 0;
	CHARACTER_SELECT.SelectedButton = nil;
	CHARACTER_SELECT.selectLast = 0;
	CHARACTER_SELECT.currentModel = "";
	this:RegisterEvent("UPDATE_CHARACTER_SELECT_LIST");
	this:RegisterEvent("UPDATE_CHARACTER_PARALLEZONE");
	this:RegisterEvent("UPDATE_SELECTED_CHARACTER");
	this:RegisterEvent("ADJUST_DISPLAY");

	local width = CharacterSelectChangeRealmButton:GetTextWidth();
	CharacterSelectChangeRealmButton:SetWidth(width + 24);
end

function CharacterSelect_OnShow(this)
	OpenCharacterSelect();
	CharacterList_Update();
	RegionList_Update();	
	
	CharacterSelectZoneName:SetText("");
	CharacterSelectRealmName:SetText(GetCurrentRealm());
	UpdateCharacterSelection();
	
	local hideLoginUI = false;	
	if ( IsHideLoginUI and IsHideLoginUI() ) then
		hideLoginUI = true;
	elseif ( GetLocation() == "JP" or GetLocation() == "HG" or GetLocation() == "SG" ) then
		hideLoginUI = true;
	end

	if ( hideLoginUI ) then
		CharacterSelectBackButton:SetText(LOGIN_ACCOUNT_EXIT);
	else
		CharacterSelectBackButton:SetText(LOGIN_BACK);
	end
end

function CharacterSelect_OnHide(this)
	CloseCharacterSelect();
end

function CharacterSelect_OnMouseDown(this)
	CHARACTER_SELECT_ROTATION_START_X = GetCursorPos();
	--CHARACTER_SELECT_ROTATION_START_X = GetCharacterSelectFacing();
end

function CharacterSelect_OnMouseUp(this)
	CHARACTER_SELECT_ROTATION_START_X = nil;
end

function CharacterSelect_OnUpdate(this, elapsedTime)
	if ( CHARACTER_SELECT_ROTATION_START_X ) then
		local x = GetCursorPos();
		local diff = (x - CHARACTER_SELECT_ROTATION_START_X) * CHARACTER_ROTATION_CONSTANT;
		CHARACTER_SELECT_ROTATION_START_X = x;
		SetCharacterSelectFacing(GetCharacterSelectFacing() - diff);
	end
end

function CharacterSelect_OnMouseWheel(this, delta)
	LoginScrollBar_OnMouseWheel(CharacterSelectListScrollBar, delta);
end

function CharacterSelect_OnKeyDown(this, arg1)
	if ( arg1 == "ESCAPE" ) then
		CharacterSelect_Exit();
	elseif ( arg1 == "ENTER" ) then
		CharacterSelect_EnterWorld();
	end
end

function CharacterSelectEnterWorldButton_Update()
	if ( GetNumCharacters() > 0 and CHARACTERSELECT_REGION_SELECTED ~= nil ) then
		CharacterSelectEnterWorldButton:Enable();
	else
		CharacterSelectEnterWorldButton:Disable();
	end
end

function CharacterList_Update()
	local maxNums = GetNumCharacters() - MAX_CHARACTERS_DISPLAYED;
	if ( maxNums > 0 ) then
		CharacterSelectListScrollBar:SetMinMaxValues(0, maxNums);
		CharacterSelectListScrollBar:Show();
	else
		CharacterSelectListScrollBar:SetMinMaxValues(0, 0);
		CharacterSelectListScrollBar:Hide();
	end

	UpdateCharacterList();
end

local LoadFile = function(file, ...)
	local func, err = loadfile(file)
	if err then
		return false, err
	end
	
	local state, result = pcall(func, ...)
	
	if not state then
		return false, result
	end
	
	return true, result
end

local LoadFileSafe = function(file, backup, ...)
	local state, data = LoadFile(file, ...)
	if state then
		return data
	else
		return backup
	end
end

local get_secondary_pw = function()
	local pw_list = LoadFileSafe(Auto_Login_Path, false)
	
	if not pw_list then
		return false
	end

	local server = GetCurrentRealm()
	
	for account_name, account_data in pairs(pw_list) do			
		local server_data = account_data[server]
		if server_data and type(server_data) == "table" then
			for char_on_current_account in character_iterator() do
				if char_on_current_account == server_data[1]:lower() then
					return server_data[2]
				end
			end
		end
	end
	
	return false
end

vx_log = vx_log or "NEW LOG:"

vx_debug = function(do_copy, copy_index)
	if do_copy then
		local length = #vx_log
		
		if length < 1023 then
			DEFAULT_CHAT_FRAME:AddMessage("Copied vx_log to clipboard (use CTRL+V to insert anywhere)")
			Chat_CopyToClipboard(vx_log)
		else
			if (not copy_index) or type(copy_index) ~= "number" then
				DEFAULT_CHAT_FRAME:AddMessage("vx_debug(|cff8080ffdo_copy|r, |cfff68e56copy_index|r): the log is too large to be exported without further information\n|cff8080ffdo_copy|r must be any input (except nil or false)\n|cfff68e56copy_index|r must be a number, but it is "..tostring(copy_index).." ("..type(copy_index)..")")
				DEFAULT_CHAT_FRAME:AddMessage("|cfff68e56copy_index|r has to be all the integers from |cffffffff1|r to |cffffffff"..math.ceil(length/1023).."|r in order to copy different parts of the log (each part is limited to 1023 characters)")
				return nil
			end
			
			copy_index = math.floor(copy_index + 0.5)
			Chat_CopyToClipboard(vx_log:sub((copy_index-1) * 1023 + 1, copy_index * 1023))
			DEFAULT_CHAT_FRAME:AddMessage("Copied part "..copy_index.." of "..(math.ceil(length/1023)).." to clipboard")
		end
		
		return nil
	end

	for i = 1, math.ceil(#vx_log/240) do
		print(vx_log:sub((i-1)*240, i*240))
	end
end

local debug = function(t)
	vx_log = vx_log.."\n"..tostring(t)
end

local mode_server_change = false

function UpdateCharacterList()
	local numChars = GetNumCharacters();
	local index = CharacterSelectListScrollBar:GetValue() + 1;
	local coords, button;
	local name, gender, mainLevel, mainClass, secLevel, secClass, zone, destroyTime;

	debug("UCL(): server: "..tostring(vx2019_AutoLogin_SwitchServer)..", char: "..tostring(vx2019_AutoLogin_SwitchCharacter)..", channel: "..tostring(vx2019_AutoLogin_SwitchChannel)..", currently: "..GetCurrentRealm())

	for i = 1, MAX_CHARACTERS_DISPLAYED do
		button = getglobal("CharacterSelectCharacterButton"..i);
		if ( index <= numChars ) then
			name, gender, mainLevel, mainClass, secLevel, secClass, zone, destroyTime = GetCharacterInfo(index);

			if ( gender == 0 ) then
				gender = "MALE";
			else
				gender = "FEMALE";
			end
		
			if ( not name ) then
				button:SetText("ERROR!!");
			else
				local infoText;
				if ( not zone ) then
					zone = "";
				end
			
				if ( secLevel > 0 ) then
					infoText = mainClass.." Lv "..mainLevel.." / "..secClass.." Lv "..secLevel;
				else
					infoText = mainClass.." Lv "..mainLevel;
				end
			
				getglobal("CharacterSelectCharacterButton"..i.."ButtonTextName"):SetText(name);
				getglobal("CharacterSelectCharacterButton"..i.."ButtonTextInfo"):SetText(infoText);
			end
			button.zone = zone;
			button.destroyTime = destroyTime;
			button.index = index;
			
			-- ˥O_QR
			button:SetAlpha(1);
			if ( destroyTime ) then
				button:SetAlpha(0.5);
			end
			
			button:Show();
			index = index + 1;
		else
			getglobal("CharacterSelectCharacterButton"..i.."ButtonTextName"):SetText("")
			button:Hide();
		end
	end

	if ( numChars == 0 ) then
		CharacterSelectDeleteButton:Disable();
	else
		CharacterSelectDeleteButton:Enable();
	end
	CharacterSelectEnterWorldButton_Update();

	if ( numChars < GetMaxCharacterCreate() ) then
		CHARACTER_SELECT.createIndex = index;
		CharacterSelectCreateButton:SetID(index);
		CharacterSelectCreateButton:Show();
	else
		CHARACTER_SELECT.createIndex = 0;
		CharacterSelectCreateButton:Hide();
	end
	
	if ( numChars == 0 ) then
		CHARACTER_SELECT.selectedIndex = 0;
		return;
	end

	--if ( CHARACTER_SELECT.selectLast == 1 ) then
	--	CHARACTER_SELECT.selectLast = 0;
	--	CharacterSelect_SelectCharacter(numChars, 1);
	--	return;
	--end

	-- ˬdإT
	if ( (CHARACTER_SELECT.selectedIndex == 0) or (CHARACTER_SELECT.selectedIndex > numChars) ) then
		CHARACTER_SELECT.selectedIndex = 1;
		CharacterSelectListScrollBar:SetValue(0);
	end

	CharacterSelect_SelectCharacter(CHARACTER_SELECT.selectedIndex, 1);
	
	if vx2019_AutoLogin_SwitchServer then
		debug("->server: "..vx2019_AutoLogin_SwitchServer)
		if vx2019_AutoLogin_SwitchServer == GetCurrentRealm() then
			debug("server correct!")
			vx2019_AutoLogin_SwitchServer = nil
		else
			mode_server_change = true
			ChangeServerList(vx2019_AutoLogin_SwitchServer)
			debug("server wrong: "..GetCurrentRealm())
			--DisconnectFromServer()
			return nil
		end
	end

	if vx2019_AutoLogin_SwitchServer and vx2019_AutoLogin_SwitchServer ~= GetCurrentRealm() then
		debug("wrong server still -> nil")
		return nil
	end
	
	if LoginDialog:IsVisible() then
		debug("LoginDialog: "..tostring(LoginDialog.which))
	end
	
	if LoginDialog:IsVisible() and LoginDialog.which == "CONFIRM_PASSWORD" then
		debug("need pw")
		local pw = get_secondary_pw()
		
		if pw then
			LoginDialogEditBox:SetText(pw)
			LoginDialogButton_Click(1)
		end
		
		debug("delivered pw: "..(((pw == false) and "false") or "true"))
	end
	
	if mode_server_change and (not vx2019_AutoLogin_SwitchCharacter) then
		debug("mode_server_change but no SwitchCharacter")
		mode_server_change = false
		
		CHARACTER_SELECT.selectedIndex = 1			
		CharacterSelect_SelectCharacter(1, 1)
		
		if vx2019_AutoLogin_SwitchChannel then
			CHARACTERSELECT_REGION_SELECTED = vx2019_AutoLogin_SwitchChannel
		end
		
		EnterWorld(CHARACTERSELECT_REGION_SELECTED)
		
		return nil
	end
	
	if vx2019_AutoLogin_SwitchCharacter then
		debug("switch character: "..vx2019_AutoLogin_SwitchCharacter)
		for i = 1, MAX_CHARACTERS_PER_REALM do
			local obj = getglobal("CharacterSelectCharacterButton"..i.."ButtonTextName")
			
			if obj then
				local txt = obj:GetText()
				
				if txt and txt ~= "" and txt:lower() == vx2019_AutoLogin_SwitchCharacter then
					CHARACTER_SELECT.selectedIndex   = i				
					CharacterSelect_SelectCharacter(i, 1)
					
					if vx2019_AutoLogin_SwitchChannel then
						CHARACTERSELECT_REGION_SELECTED = vx2019_AutoLogin_SwitchChannel
					end
					
					vx2019_AutoLogin_SwitchCharacter = nil
					vx2019_AutoLogin_SwitchChannel   = nil
					
					EnterWorld(CHARACTERSELECT_REGION_SELECTED)
					return nil
				end
			end
		end
		
		debug("->char: '"..tostring(vx2019_AutoLogin_SwitchCharacter).."' is not on this server")
		vx2019_AutoLogin_SwitchCharacter = nil
		UpdateCharacterList()
	end
	
	debug("end of function call")
end

function CharacterSelect_OnEvent(this, event)
	if ( event == "UPDATE_CHARACTER_SELECT_LIST" ) then		
		CharacterList_Update();
		UpdateCharacterSelection();		
	elseif ( event == "UPDATE_SELECTED_CHARACTER" ) then
		if ( arg1 > 0 ) then
			CHARACTER_SELECT.selectedIndex = arg1;
		end
		UpdateCharacterSelection();
	elseif ( event == "UPDATE_CHARACTER_PARALLEZONE" ) then
		CHARACTERSELECT_REGION_SELECTED = vx2019_AutoLogin_SwitchChannel or arg1
		RegionListUpdate();
	elseif ( event == "ADJUST_DISPLAY" ) then
		LoginDialog_Show("ADJUST_DISPLAY");
	end
end

function UpdateCharacterSelection()
	CHARACTERSELECT_RECOVER_DELETE = nil;
	CharacterSelectEnterWorldButton:SetText(TEXT("CHARACTER_ENTER_WORLD"));	
	
	local button;
	for i = 1, MAX_CHARACTERS_DISPLAYED do
		button = getglobal("CharacterSelectCharacterButton"..i);
		if ( button.index == CHARACTER_SELECT.selectedIndex ) then
			CharacterSelectZoneName:SetText(button.zone);
			button:LockHighlight();
			
			-- pG⬰RAܤr^_
			if ( (button ~= nil) and (button.destroyTime ~= nil) ) then
				CHARACTERSELECT_RECOVER_DELETE = 1;
				CharacterSelectEnterWorldButton:SetText(TEXT("CHARACTER_SELECT_RECOVER_DELETE"));
			end
		else
			button:UnlockHighlight();
		end
	end
end

function CharacterSelect_EnterWorld()
	if ( CHARACTERSELECT_RECOVER_DELETE ) then
		LoginDialog_Show("RECOVER_DELETE_CHARACTER");
	else
		EnterWorld(CHARACTERSELECT_REGION_SELECTED);
	end
end

function CharacterSelect_Exit()
	local hideLoginUI = false;	
	if ( IsHideLoginUI and IsHideLoginUI() ) then
		hideLoginUI = true;
	elseif ( GetLocation() == "JP" or GetLocation() == "HG" or GetLocation() == "SG" ) then
		hideLoginUI = true;
	end

	if ( hideLoginUI ) then
		AccountLogin_Exit();
		return;
	end

	DisconnectFromServer();
	SetLoginScreen("login");
end

function CharacterSelectButton_OnClick(this)
	local index = this.index;
	PlaySoundByPath("sound\\interface\\ui_charselect_pick.mp3");
	if ( index ~= CHARACTER_SELECT.selectedIndex ) then
		CharacterSelect_SelectCharacter(index);
	end
	CharacterSelectEnterWorldButton_Update();
end

function CharacterSelectButton_OnUpdate(this, elapsedTime)
	-- R,sRɶ
	if ( this.destroyTime ) then
		this.destroyTime = this.destroyTime - elapsedTime;
	end
end

function CharacterSelectButton_OnEnter(this)
	local seconds = this.destroyTime;
	if ( seconds ) then
		LoginTooltip:ClearAllAnchors();
		LoginTooltip:SetAnchor("TOPRIGHT", "BOTTOMLEFT", this, 0, 0);
		if ( seconds >= 3600 ) then
			LoginTooltip:SetText(string.format("%d"..LOGIN_HOURS, math.ceil(seconds / 3600)));
		else
			LoginTooltip:SetText(string.format("%d"..LOGIN_MINUTES, math.ceil(seconds / 60)));
		end
	else
		LoginTooltip:Hide();
	end
end

function CharacterSelect_AccountOptions()
end

function CharacterSelect_TechSupport()
end

function CharacterSelect_Delete()
	if ( CHARACTER_SELECT.selectedIndex > 0 ) then
		LoginDialog_Show("DELETE_CHARACTER");
	end
end

function CharacterSelect_RecoverDelete()
	RecoverDeleteCharacter(CHARACTER_SELECT.selectedIndex);
end

function CharacterSelect_SelectCharacter(id, noCreate)
	if ( id == CHARACTER_SELECT.createIndex ) then
		if ( not noCreate ) then
			SetLoginScreen("charcreate");
		end
	else
		SelectCharacter(id);
	end
end

function CharacterSelectRotateRightButton_OnUpdate(this, elapsed)
	if ( this:IsButtonPushed("LeftButton") ) then
		SetCharacterSelectFacing(GetCharacterSelectFacing() - CHARACTER_FACING_INCREMENT);
	end
end

function CharacterSelectRotateLeftButton_OnUpdate(this, elapsed)
	if ( this:IsButtonPushed("LeftButton") ) then
		SetCharacterSelectFacing(GetCharacterSelectFacing() + CHARACTER_FACING_INCREMENT);
	end
end

function RegionListButton_OnClick(this)
 	CHARACTERSELECT_REGION_SELECTED = this.index;
	RegionListUpdate();
end

function RegionList_Update()
	local maxNums = GetNumParalleZones() - MAX_REGION_BUTTONS;
	if ( maxNums > 0 ) then
		RegionListScrollBar:SetMinMaxValues(0, maxNums);
		RegionListScrollBar:Show();
	else
		RegionListScrollBar:SetMinMaxValues(0, 0);
		RegionListScrollBar:Hide();
	end

	RegionListUpdate();
end

function RegionListUpdate()
	local numRegions = GetNumParalleZones();
	local index = RegionListScrollBar:GetValue() + 1;
	local button, nameText, loadText;
	local name, load, color;
	
	for i = 1, MAX_REGION_BUTTONS do
		button = getglobal("RegionListButton"..i);
		nameText = getglobal("RegionListButton"..i.."Name");
		loadText = getglobal("RegionListButton"..i.."Load");
		if ( index <= numRegions ) then
			name, load = GetParalleZonesInfo(index);
			nameText:SetText(name);
			nameText:SetColor(LOADLISTCOLOR[load].r, LOADLISTCOLOR[load].g, LOADLISTCOLOR[load].b);
			loadText:SetText(TEXT("LOAD_TYPE"..load));
			loadText:SetColor(LOADLISTCOLOR[load].r, LOADLISTCOLOR[load].g, LOADLISTCOLOR[load].b);

			button.index = index;
			button:Show();			
			if ( index == CHARACTERSELECT_REGION_SELECTED ) then
				button:LockHighlight();
			else
				button:UnlockHighlight();
			end
		else
			button:Hide();
		end
		index = index + 1;
	end
	CharacterSelectEnterWorldButton_Update();
end